#include "Order_HLine.mqh"

struct ORDER_HLINES
{
private:
  const string Prefix;

  ORDER_HLINE HLines[];

  static int GetDigits( double Price )
  {
    int Res = 0;

    while ((bool)(Price = ::NormalizeDouble(Price - (int)Price, 8)))
    {
      Price *= 10;

      Res++;
    }

    return(Res);
  }

public:
  ORDER_HLINES( const string sPrefix = "Tester_" ) : Prefix(sPrefix)
  {
    this.Set();
  }

  int GetPos( const string& Name ) const
  {
    const int Size = ::ArraySize(this.HLines);
    int Pos = 0;

    while ((Pos < Size) && (this.HLines[Pos].Name != Name))
      Pos++;

    return(Pos);
  }

  int Set( const long chartID = 0 )
  {
    static const string Types[] = {"buy", "sell", "buy limit", "sell limit", "buy stop", "sell stop", "balance"};

    const color ColorSLTP = (color)::ChartGetInteger(chartID, CHART_COLOR_STOP_LEVEL);
    const int Total = OrdersTotal();
    ::ArrayResize(this.HLines, Total * 3);
    int Amount = 0;

    for (int i = 0; i < Total; i++)
      if (OrderSelect(i, SELECT_BY_POS))
      {
        const int VolumeDigits = ORDER_HLINES::GetDigits(::SymbolInfoDouble(OrderSymbol(), SYMBOL_VOLUME_STEP));

        this.HLines[Amount++].Set(this.Prefix + (string)OrderTicket(),
                                  "#" + (string)OrderTicket() + " " + Types[OrderType()] + " " +
                                  ::DoubleToString(OrderLots(), VolumeDigits),
                                  OrderOpenPrice(), clrGreen);

        if (OrderTakeProfit())
          this.HLines[Amount++].Set(this.Prefix + (string)OrderTicket() + "_TP",
                                    "#" + (string)OrderTicket() + " tp",
                                    OrderTakeProfit(), ColorSLTP);

        if (OrderStopLoss())
          this.HLines[Amount++].Set(this.Prefix + (string)OrderTicket() + "_SL",
                                    "#" + (string)OrderTicket() + " sl",
                                    OrderStopLoss(), ColorSLTP);
      }

    return(::ArrayResize(this.HLines, Amount));
  }

  bool IsMyName( const string &Str ) const
  {
    return(::StringSubstr(Str, 0, ::StringLen(this.Prefix)) == this.Prefix);
  }

  void Sync( const long chartID = 0 )
  {
    const int Size = ArraySize(this.HLines);

    for (int i = ::ObjectsTotal(chartID, 0, OBJ_HLINE) - 1; i >= 0; i--)
    {
      const string Name = ::ObjectName(chartID, i, 0, OBJ_HLINE);

      if (this.IsMyName(Name))
      {
        const int Pos = this.GetPos(Name);

        if (Pos < Size)
          this.HLines[Pos].Modify(chartID);
        else
          ::ObjectDelete(chartID, Name);
      }
    }

    for (int i = 0; i < Size; i++)
      this.HLines[i].Create(chartID);

    return;
  }
};